
/***********************************************

	pulgen.c : Pulse Generator on COM Port

	2010-03-10
	Osamu Tamura @ Recursion Co., Ltd.

************************************************/

#define	_CRT_SECURE_NO_WARNINGS
#include <windows.h>
#include <stdio.h>
#include <string.h>
#include <conio.h>

static const char	*title	= "\n    Pulse Generator by Osamu Tamura";
static const char	*usage	=   "     usage: pulgen com_port_number\n";
static const char	*help	=   "       (freq:arrows, quit:space)\n";

typedef struct {
	DWORD	baud;
	BYTE	parity;
	BYTE	stop;
	WORD	code;
	char	*frequency;
} COMDCB;

#define	MAX_FREQ	30

static const COMDCB ComDcb[MAX_FREQ]	= {
{	CBR_110,	EVENPARITY,	TWOSTOPBITS,	0x1c,	"   18" },
{	CBR_300,	NOPARITY,	ONESTOPBIT,		0xf0,	"   30" },
{	CBR_300,	EVENPARITY,	TWOSTOPBITS,	0x1c,	"   50" },
{	CBR_600,	NOPARITY,	ONESTOPBIT,		0xf0,	"   60" },
{	CBR_600,	EVENPARITY,	TWOSTOPBITS,	0x1c,	"  100" },
{	CBR_1200,	NOPARITY,	ONESTOPBIT,		0xf0,	"  120" },
{	CBR_300,	NOPARITY,	ONESTOPBIT,		0x55,	"  150" },
{	CBR_1200,	EVENPARITY,	TWOSTOPBITS,	0x1c,	"  200" },
{	CBR_2400,	NOPARITY,	ONESTOPBIT,		0xf0,	"  240" },
{	CBR_600,	NOPARITY,	ONESTOPBIT,		0x55,	"  300" },
{	CBR_2400,	EVENPARITY,	TWOSTOPBITS,	0x1c,	"  400" },
{	CBR_4800,	NOPARITY,	ONESTOPBIT,		0xf0,	"  480" },
{	CBR_1200,	NOPARITY,	ONESTOPBIT,		0x55,	"  600" },
{	CBR_4800,	EVENPARITY,	TWOSTOPBITS,	0x1c,	"  800" },
{	CBR_9600,	NOPARITY,	ONESTOPBIT,		0xf0,	"  960" },
{	CBR_2400,	NOPARITY,	ONESTOPBIT,		0x55,	" 1200" },
{	CBR_14400,	NOPARITY,	ONESTOPBIT,		0xf0,	" 1440" },
{	CBR_9600,	EVENPARITY,	TWOSTOPBITS,	0x1c,	" 1600" },
{	CBR_19200,	NOPARITY,	ONESTOPBIT,		0xf0,	" 1920" },
{	CBR_4800,	NOPARITY,	ONESTOPBIT,		0x55,	" 2400" },
{	CBR_19200,	EVENPARITY,	TWOSTOPBITS,	0x1c,	" 3200" },
{	CBR_38400,	NOPARITY,	ONESTOPBIT,		0xf0,	" 3840" },
{	CBR_9600,	NOPARITY,	ONESTOPBIT,		0x55,	" 4800" },
{	CBR_56000,	NOPARITY,	ONESTOPBIT,		0xf0,	" 5600" },
{	CBR_57600,	NOPARITY,	ONESTOPBIT,		0xf0,	" 5760" },
{	CBR_38400,	EVENPARITY,	TWOSTOPBITS,	0x1c,	" 6400" },
{	CBR_14400,	NOPARITY,	ONESTOPBIT,		0x55,	" 7200" },
{	CBR_19200,	NOPARITY,	ONESTOPBIT,		0x55,	" 9600" },
{	CBR_115200,	NOPARITY,	ONESTOPBIT,		0xf0,	"11520" },
{	CBR_38400,	NOPARITY,	ONESTOPBIT,		0x55,	"19200" }
};

#define	LEN_DIV		50
#define	MAX_SBUF	(CBR_115200/LEN_DIV)

static char			data[MAX_SBUF];

//////////////////////////////////////////////////////////////

int main( int argc, char *argv[] )
{
	DCB				dcb;
	HANDLE			hCom;
	HANDLE			hEvent;
	OVERLAPPED		ol;
	int				index, length;
	BOOL			bQuit;

	puts( title );

	if( argc!=2 ) {
		puts( usage );
		return -1;
	}

	sprintf( data, "\\.\\COM%s", argv[1] );
	hCom = CreateFile( data,
							 GENERIC_WRITE,
							 0,
							 NULL,
							 OPEN_EXISTING,
							 FILE_FLAG_OVERLAPPED,
							 NULL
						 );
	if( hCom==INVALID_HANDLE_VALUE )
		return -2;

	hEvent = CreateEvent( NULL, FALSE, FALSE, NULL );
	if( hEvent==NULL )
		return -3;

	ZeroMemory( &ol, sizeof(ol) );
	ol.hEvent = hEvent;

	if( !GetCommState( hCom, &dcb ) ) {
		CloseHandle( hCom );
		return -4;
	}

	puts( help );

	index	= 13;

	dcb.ByteSize	= 8;
	dcb.BaudRate	= ComDcb[index].baud;
	dcb.Parity		= ComDcb[index].parity;
	dcb.StopBits	= ComDcb[index].stop;
	SetCommState( hCom, &dcb );

	length	= dcb.BaudRate / LEN_DIV;
	memset( data, ComDcb[index].code, length );
	printf( "\t%s Hz\n", ComDcb[index].frequency );

	for( bQuit=FALSE; !bQuit; ) {

		ResetEvent( hEvent );
		WriteFile( hCom, data, length, NULL, &ol );

		while( WaitForSingleObject(hEvent,200)!=WAIT_OBJECT_0 ) {
			int		c, count;

			if( !_kbhit() )
				continue;

			c = _getch();
			if( c==0xe0 )
				c	= _getch()==0x48? '+':'-';

			count	= index;
			switch( c ) {
				case '+':
				case '=':	if( count<(MAX_FREQ-1) )
								count++;
							break;
				case '_':
				case '-':	if( count>0 )
								count--;
							break;
				case ' ':
				case 'q':
				case 'Q':	bQuit	= TRUE;
				default:
							break;
			}

			if( index!=count ) {
				index	= count;

				CancelIo( hCom );

				dcb.ByteSize	= 8;
				dcb.BaudRate	= ComDcb[index].baud;
				dcb.Parity		= ComDcb[index].parity;
				dcb.StopBits	= ComDcb[index].stop;
				if( !SetCommState( hCom, &dcb ) )
					Beep( 800, 200 );

				length	= dcb.BaudRate / LEN_DIV;
				memset( data, ComDcb[index].code, length );
				printf( "\t%s Hz\n", ComDcb[index].frequency );
			}
		}
	}

	CancelIo( hCom );
	CloseHandle( hEvent );
	CloseHandle( hCom );
	return 0;
}

/////////////////////// End Of File ///////////////////////
